/* *****************************************************************************
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#ifndef CUBESIM_H
#define CUBESIM_H

class CubeBox;
class Cube;

/**
* Class for holding information about a cube.
* for example to undo a move or for simulate a move from the computerplayer
*
* @short Class for holding information about a cube. 
*/  
class CubeSim
{
   public:
	CubeSim(int owner=0,int value=1,int max=2);
   	void setOwner(int newOwner);
	void setValue(int newValue);
	void setMax(int max);
	/**
	* increases value of this Cube and sets owner to 'newOwner'
	*/
	void increase(int newOwner);
	void decrease();
	int getValue();
	int getOwner();
	int getMax();
	/**
	* returns TRUE if cube's value is bigger than max
	*/
	bool overMax();
		
   private:
	int owner;
  	int value;
	int max;  
};


/**
* Class for storing information about the playingfield, for example to undo a move 
* or computing next move for the computerplayer 
*
* @short Class which holds a simulated playingfield
*/
class CubeSimBox
{
   public:
	CubeSimBox(int size=1);
   	CubeSimBox(CubeSimBox *);
	CubeSimBox(CubeSimBox&);
	CubeSimBox(CubeBox *);
	~CubeSimBox();
	/**
	* returns cube at (row,column)
	*/
	CubeSim& cube(int row,int column);
	void copy(CubeBox*);
	/**
	* simulates a move at cube(row,column) from player 'player'
	*/
	bool simulateMove(int row, int column,int player);
	/**
	* assesses field for player 'player'
	*/
	int assessField(int player);
	bool playerWon(int player);
	int size;
			
   private:
	
	CubeSim **cubes;
	
	int maxValue(int row,int column);
	/**
	* increases all neighbourcubes to cube(row,column) and sets their owner to 
	* 'player'
	*/
	void increaseNeighbours(int player, int row, int column);
	
};




#endif // CUBESIM_H
